@@ -6,7 +6,7 @@ import sys, os
 
 try:
     import setuptools
-except ModuleNotFoundError:
+except ImportError:
     pass
 
 from distutils.core import setup, Extension
@@ -254,7 +254,7 @@ else:
 setup (name = "libxml2-python",
        # On *nix, the version number is created from setup.py.in
        # On windows, it is set by configure.js
-       version = "2.9.13",
+       version = "2.9.14",
        description = descr,
        author = "Daniel Veillard",
        author_email = "veillard@redhat.com",
